#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QTimer>
#include <QPainter>
#include <QFont>
#include <QMouseEvent>
#include <QToolTip>

#define CHIPX       14
#define CHIPY       14
#define PINWIDTH    48
#define PINHEIGHT   48
#define MAXINFO     0x100
#define MAXNAME     0x10

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    QTimer::singleShot(0, this, SLOT(onInit()));
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::onInit()
{
    setFixedSize(CHIPX * PINWIDTH, CHIPY * PINHEIGHT);

    char gname[][MAXNAME] = {"USB", "GND", "SDRAM", "GPIO", "JTAG", "RF",
        "UARTL", "UARTF", "LDO", "EPHY", "MICS", "PLL", "SPI", "I2C", "VCC"};
    enum GROUPS {USB, GND, SDRAM, GPIO, JTAG, RF,
        UARTL, UARTF, LDO, EPHY, MICS, PLL, SPI, I2C, VCC};
    QMap<QString, int> gmap;
    for(unsigned int i = 0; i < sizeof(gname) / MAXNAME; i++)
        gmap[QString(gname[i])] = i;

    enum PINS { A = 0, B = 14, C = 28, D = 42, E = 56, F = 70, G = 84, H = 98,
        J = 112, K = 126, L = 140, M = 154, N = 168, P = 182};
    group[USB] = QColor(200, 100, 20);
    group[GND] = QColor(0, 0, 0);
    group[SDRAM] = QColor(170, 170, 30);
    group[GPIO] = QColor(30, 170, 170);
    group[JTAG] = QColor(170, 30, 170);
    group[RF] = QColor(0, 200, 0);
    group[UARTL] = QColor(0, 0, 200);
    group[UARTF] = QColor(200, 140, 140);
    group[LDO] = QColor(140, 200, 140);
    group[EPHY] = QColor(140, 140, 200);
    group[MICS] = QColor(100, 0, 0);
    group[PLL] = QColor(0, 100, 0);
    group[SPI] = QColor(0, 0, 100);
    group[I2C] = QColor(120, 120, 120);
    group[VCC] = QColor(200, 0, 0);

    map.clear();
    map[A + 11] = "JTAG:JTAG_TRST_N,GPIO:I ,IPU:4 mA:JTAG TRST (active low)";
    map[A + 12] = "JTAG:JTAG_TCLK,GPIO:I, IPD:4 mA:JTAG TCLK";
    map[A + 13] = "JTAG:JTAG_TMS,GPIO:I, IPD:4 mA:JTAG TMS";
    map[A + 14] = "JTAG:JTAG_TDI,GPIO:I, IPD:4 mA:JTAG TDI";
    map[B + 11] = "JTAG:JTAG_TDO,GPIO:O, IP:4 mA:JTAG TDO";

    map[P + 3] = "UARTL:RXD2,GPIO:I, IPD:4 mA:UART Lite RXD";
    map[N + 2] = "UARTL:TXD2,GPIO:O, IPD:4 mA:UART Lite TXD";

    map[N + 3] = "UARTF:RXD,GPIO:I, IPD:4 mA:UART RXD";
    map[M + 3] = "UARTF:RIN,GPIO:I, IPD:4 mA:UART RIN";
    map[K + 4] = "UARTF:CTS_N,GPIO:I, IPD:4 mA:UART CTS_N";
    map[L + 3] = "UARTF:DSR_N,GPIO:I, IPD:4 mA:UART DSR_N";
    map[J + 4] = "UARTF:DCD_N,GPIO:I, IPD:4 mA:UART DCD_N";
    map[K + 2] = "UARTF:TXD,GPIO:O, IPD:4 mA:UART TXD";
    map[N + 4] = "UARTF:DTR_N,GPIO:O, IPD:4 mA:UART DTR_N";
    map[P + 2] = "UARTF:RTS_N,GPIO:O, IPD:4 mA:UART RTS_N";

    map[M + 2] = "SPI:SPI_MISO,GPIO:I, IPD:4mA:SPI Master In Slave Out";
    map[J + 3] = "SPI:SPI_MOSI,GPIO:O, IPD:4mA:SPI Master Out Slave In";
    map[N + 1] = "SPI:SPI_CLK,GPIO:O, IPD:4mA:SPI Clock";
    map[P + 1] = "SPI:SPI_CS0,GPIO:O, IPD:4mA:SPI Chip Select 0";
    map[L + 2] = "SPI:SPI_CS1,GPIO:O, IPD:4mA:SPI Chip Select 1";

    map[B + 14] = "I2C:I2C_SCLK,GPIO:I/O, IPU:8mA:I2C Clock";
    map[B + 13] = "I2C:I2C_SD,GPIO:O, IPU:8mA:I2C Data";

    map[B + 12] = "GPIO:GPIO0:I/O, IPD:8mA:GPIO0";

    map[K + 3] = "EPHY:EPHY_LED0_N,GPIO:O, IPD:4mA:10/100 Phy Port #0 Activity LED";
    map[L + 1] = "EPHY:EPHY_LED1_N,GPIO:O, IPD:4mA:10/100 Phy Port #1 Activity LED";
    map[L + 4] = "EPHY:EPHY_LED2_N,GPIO:O, IPD:4mA:10/100 Phy Port #2 Activity LED";
    map[M + 1] = "EPHY:EPHY_LED3_N,GPIO:O, IPD:4mA:10/100 Phy Port #3 Activity LED";
    map[M + 4] = "EPHY:EPHY_LED4_N,GPIO:O, IPD:4mA:10/100 Phy Port #4 Activity LED";
    map[P + 4] = "EPHY:EPHY_REF_RES:A::Connects to an external resistor to provide accurate bias current";
    map[L + 5] = "EPHY:EPHY_RXN_P0:I::10/100 PHY Port #0 RXN";
    map[M + 5] = "EPHY:EPHY_RXP_P0:I::10/100 PHY Port #0 RXP";
    map[N + 5] = "EPHY:EPHY_TXN_P0:O::10/100 PHY Port #0 TXN";
    map[P + 5] = "EPHY:EPHY_TXP_P0:O::10/100 PHY Port #0 TXP";
    map[L + 6] = "EPHY:EPHY_RXN_P1:I::10/100 PHY Port #1 RXN";
    map[M + 6] = "EPHY:EPHY_RXP_P1:I::10/100 PHY Port #1 RXP";
    map[N + 6] = "EPHY:EPHY_TXN_P1:O::10/100 PHY Port #1 TXN";
    map[P + 6] = "EPHY:EPHY_TXP_P1:O::10/100 PHY Port #1 TXP";
    map[N + 7] = "EPHY:EPHY_RXN_P2:I::10/100 PHY Port #2 RXN";
    map[P + 7] = "EPHY:EPHY_RXP_P2:I::10/100 PHY Port #2 RXP";
    map[M + 7] = "EPHY:EPHY_TXN_P2:O::10/100 PHY Port #2 TXN";
    map[L + 7] = "EPHY:EPHY_TXP_P2:O::10/100 PHY Port #2 TXP";
    map[M + 8] = "EPHY:EPHY_RXN_P3:I::10/100 PHY Port #3 RXN";
    map[L + 8] = "EPHY:EPHY_RXP_P3:I::10/100 PHY Port #3 RXP";
    map[N + 8] = "EPHY:EPHY_TXN_P3:O::10/100 PHY Port #3 TXN";
    map[P + 8] = "EPHY:EPHY_TXP_P3:O::10/100 PHY Port #3 TXP";
    map[N + 9] = "EPHY:EPHY_RXN_P4:I::10/100 PHY Port #4 RXN";
    map[P + 9] = "EPHY:EPHY_RXP_P4:I::10/100 PHY Port #4 RXP";
    map[M + 9] = "EPHY:EPHY_TXN_P4:O::10/100 PHY Port #4 TXN";
    map[L + 9] = "EPHY:EPHY_TXP_P4:O::10/100 PHY Port #4 TXP";

    map[C + 10] = "MICS:PORST_N:I, IPU:2mA:Power On Reset";
    map[K + 1] = "MICS:WLAN_LED_N:O, IPD:4mA:WLAN Activity LED";

    map[N + 11] = "USB:UPHY0_VDDA_V33A:P::3.3V USB PHY analog power supply";
    map[P + 13] = "USB:UPHY0_VDDL_V12D:P::1.2V USB PHY digital power supply";
    map[P + 11] = "USB:UPHY0_VRES:I/O::Connects to an external 8.2K Ohm resistor for band-gap reference circuit.";
    map[N + 12] = "USB:UPHY0_PADM:I/O::USB data pin Data-";
    map[P + 12] = "USB:UPHY0_PADP:I/O::USB data pin Data+";

    map[F + 14] = "SDRAM:MD15:I/O:4/8 mA:SDRAM Data Bit #15";
    map[G + 13] = "SDRAM:MD14:I/O:4/8 mA:SDRAM Data Bit #14";
    map[F + 13] = "SDRAM:MD13:I/O:4/8 mA:SDRAM Data Bit #13";
    map[G + 12] = "SDRAM:MD12:I/O:4/8 mA:SDRAM Data Bit #12";
    map[G + 11] = "SDRAM:MD11:I/O:4/8 mA:SDRAM Data Bit #11";
    map[E + 14] = "SDRAM:MD10:I/O:4/8 mA:SDRAM Data Bit #10";
    map[E + 13] = "SDRAM:MD9:I/O:4/8 mA:SDRAM Data Bit #9";
    map[F + 12] = "SDRAM:MD8:I/O:4/8 mA:SDRAM Data Bit #8";
    map[E + 12] = "SDRAM:MD7:I/O:4/8 mA:SDRAM Data Bit #7";
    map[F + 11] = "SDRAM:MD6:I/O:4/8 mA:SDRAM Data Bit #6";
    map[E + 11] = "SDRAM:MD5:I/O:4/8 mA:SDRAM Data Bit #5";
    map[D + 14] = "SDRAM:MD4:I/O:4/8 mA:SDRAM Data Bit #4";
    map[D + 13] = "SDRAM:MD3:I/O:4/8 mA:SDRAM Data Bit #3";
    map[D + 12] = "SDRAM:MD2:I/O:4/8 mA:SDRAM Data Bit #2";
    map[D + 11] = "SDRAM:MD1:I/O:4/8 mA:SDRAM Data Bit #1";
    map[E + 10] = "SDRAM:MD0:I/O:4/8 mA:SDRAM Data Bit #0";
    map[L + 13] = "SDRAM:MA12:I/O:4/8 mA:SDRAM Address Bit #12";
    map[K + 12] = "SDRAM:MA11:I/O:4/8 mA:SDRAM Address Bit #11";
    map[L + 14] = "SDRAM:MA10:I/O:4/8 mA:SDRAM Address Bit #10";
    map[J + 11] = "SDRAM:MA9:I/O:4/8 mA:SDRAM Address Bit #9";
    map[K + 13] = "SDRAM:MA8:I/O:4/8 mA:SDRAM Address Bit #8";
    map[K + 14] = "SDRAM:MA7:I/O:4/8 mA:SDRAM Address Bit #7";
    map[J + 12] = "SDRAM:MA6:I/O:4/8 mA:SDRAM Address Bit #6";
    map[J + 13] = "SDRAM:MA5:I/O:4/8 mA:SDRAM Address Bit #5";
    map[J + 14] = "SDRAM:MA4:I/O:4/8 mA:SDRAM Address Bit #4";
    map[H + 11] = "SDRAM:MA3:I/O:4/8 mA:SDRAM Address Bit #3";
    map[H + 12] = "SDRAM:MA2:I/O:4/8 mA:SDRAM Address Bit #2";
    map[H + 13] = "SDRAM:MA1:I/O:4/8 mA:SDRAM Address Bit #1";
    map[G + 14] = "SDRAM:MA0:I/O:4/8 mA:SDRAM Address Bit #0";
    map[M + 14] = "SDRAM:MBA1:I/O:4/8 mA:SDRAM MBA #1";
    map[L + 12] = "SDRAM:MBA0:I/O:4/8 mA:SDRAM MBA #0";
    map[L + 11] = "SDRAM:MRAS_N:I/O:4/8 mA:SDRAM MRAS_N";
    map[C + 12] = "SDRAM:MCAS_N:I/O:4/8 mA:SDRAM MCAS_N";
    map[C + 13] = "SDRAM:MWE_N:I/O:4/8 mA:SDRAM M_WE_N";
    map[H + 14] = "SDRAM:MCLK:I/O:4/8 mA:SDRAM MCLK";
    map[C + 11] = "SDRAM:MCKE:I/O:4/8 mA:SDRAM MCKE";
    map[M + 13] = "SDRAM:MDQM1:I/O:4/8 mA:SDRAM MDQM #1";
    map[J + 10] = "SDRAM:MDQM0:I/O:4/8 mA:SDRAM MDQM #0";
    map[K + 11] = "SDRAM:MCS0_N:I/O:4/8 mA:SDRAM MCS0_N";
    map[C + 14] = "SDRAM:MCS1_N:I/O:4/8 mA:SDRAM MCS1_N";

    map[G + 2] = "LDO:LDO_V18A:P::1.8V power input for internal MOS";
    map[G + 1] = "LDO:VOUT_1P2:P::1.2V regulation output";
    map[G + 3] = "LDO:LDOSEL:I::Internal/External LDO select. Default: floating, use internal. Tied to 3.3: use externally";
    map[H + 3] = "LDO:EXT_LDO_1P2:P::Gate drive for external BJT";
    map[H + 2] = "LDO:DCDC_V33A:P::3.3V agalog power";
    map[G + 4] = "LDO:COMP:A::This pin is the error amplifier output and combines with the FB pin to compensate the voltage control";
    map[H + 4] = "LDO:FB:A::Programmable feedback reference voltage for SW regulator and compensation network of the error amplifier";
    map[H + 1] = "LDO:UGATE:A::Gate drive for external upper MOSFET";
    map[J + 1] = "LDO:LGATE:A::Gate driver for external lower MOSFET";
    map[J + 2] = "LDO:DCDC_V33D:P::3.3V power supply only for gate driver of SW(lpeak < 200mA; lavg < 20mA)";

    map[B + 10] = "PLL:PLL_DVDD_V12D:P::1.2V digital power supply to PLL";
    map[A + 10] = "PLL:PLL_AVDD_V12A:P::1.2V analog power supply to PLL";

    map[A + 2] = "RF:WL_RF0_2G_INP:I::2.4GHz RX0 input(positive)";
    map[A + 3] = "RF:WL_RF0_2G_INN:I::2.4GHz RX0 input(negative)";
    map[B + 1] = "RF:WL_RF0_PA_V33P:P::3.3V supply for RF channel 0";
    map[C + 1] = "RF:WL_RF0_2G_OUTP:O::2.4GHz TX output(negative)";
    map[D + 1] = "RF:WL_RF0_2G_OUTN:O::2.4GHz TX0 PA output(negative)";
    map[E + 1] = "RF:WL_RF0_PA_V33N:P::3.3V supply for RF channle 0";
    map[E + 3] = "RF:WL_RF0_PA1_INP:P::3.3V supply for RF0 PA1";
    map[B + 5] = "RF:WL_RF0_IF_V12A:P::1.2V supply for IF0";
    map[B + 3] = "RF:WL_RF0_RF_V12A:P::1.2V supply for RF0";
    map[B + 9] = "RF:WL_ADC_V12:P::1.2V supply for ADC analog blocks";
    map[A + 5] = "RF:WL_RF_BB1_V12A:P::1.2V supply for analog baseband";
    map[C + 9] = "RF:WL_RF_BB2_V12A:P::1.2V supply for analog baseband";
    map[B + 8] = "RF:WL_BG_RES_12K:I/O::External reference resistor(12K ohm)";
    map[A + 9] = "RF:WL_BG_V33A:P::3.3V supply for band gap reference";
    map[C + 8] = "RF:WL_LDORF_OUT_V12:O::LDO 1.2V 200mA output for RF core";
    map[C + 7] = "RF:WL_LDOPLL_OUT_V12:O::LDO 1.2V 200mA output for PLL core";
    map[A + 8] = "RF:WL_LDORF_IN_VX:I::LDO 1.5~2V 300mA input for RF core and PPL";
    map[A + 7] = "RF:WL_PLL_X1:I::Crystal oscillator input";
    map[B + 7] = "RF:WL_PLL_X2:O::Crystal oscillator output";
    map[B + 6] = "RF:WL_PLL_V12A:P::1.2V supply for PLL";
    map[A + 6] = "RF:WL_PLL_VC_CAP:I/O::PLL external loop filter";
    map[C + 6] = "RF:WL_VCO_VCO_V12A:P::1.2V supply for VCO output buffer";

    map[A + 1] = "GND:GND:G::Ground pin";
    map[A + 4] = "GND:GND:G::Ground pin";
    map[B + 2] = "GND:GND:G::Ground pin";
    map[B + 4] = "GND:GND:G::Ground pin";
    map[C + 2] = "GND:GND:G::Ground pin";
    map[C + 3] = "GND:GND:G::Ground pin";
    map[C + 4] = "GND:GND:G::Ground pin";
    map[C + 5] = "GND:GND:G::Ground pin";
    map[D + 2] = "GND:GND:G::Ground pin";
    map[D + 3] = "GND:GND:G::Ground pin";
    map[D + 4] = "GND:GND:G::Ground pin";
    map[D + 5] = "GND:GND:G::Ground pin";
    map[D + 6] = "GND:GND:G::Ground pin";
    map[D + 7] = "GND:GND:G::Ground pin";
    map[D + 8] = "GND:GND:G::Ground pin";
    map[D + 9] = "GND:GND:G::Ground pin";
    map[E + 2] = "GND:GND:G::Ground pin";
    map[E + 4] = "GND:GND:G::Ground pin";
    map[E + 5] = "GND:GND:G::Ground pin";
    map[E + 6] = "GND:GND:G::Ground pin";
    map[E + 7] = "GND:GND:G::Ground pin";
    map[E + 8] = "GND:GND:G::Ground pin";
    map[E + 9] = "GND:GND:G::Ground pin";
    map[F + 1] = "GND:GND:G::Ground pin";
    map[F + 2] = "GND:GND:G::Ground pin";
    map[F + 3] = "GND:GND:G::Ground pin";
    map[F + 4] = "GND:GND:G::Ground pin";
    map[F + 6] = "GND:GND:G::Ground pin";
    map[F + 7] = "GND:GND:G::Ground pin";
    map[F + 8] = "GND:GND:G::Ground pin";
    map[G + 6] = "GND:GND:G::Ground pin";
    map[G + 7] = "GND:GND:G::Ground pin";
    map[G + 8] = "GND:GND:G::Ground pin";
    map[H + 6] = "GND:GND:G::Ground pin";
    map[H + 7] = "GND:GND:G::Ground pin";
    map[H + 8] = "GND:GND:G::Ground pin";
    map[J + 6] = "GND:GND:G::Ground pin";
    map[J + 7] = "GND:GND:G::Ground pin";
    map[J + 8] = "GND:GND:G::Ground pin";
    map[J + 9] = "GND:GND:G::Ground pin";
    map[K + 9] = "GND:GND:G::Ground pin";
    map[K + 10] = "GND:GND:G::Ground pin";
    map[L + 10] = "GND:GND:G::Ground pin";
    map[M + 10] = "GND:GND:G::Ground pin";
    map[M + 11] = "GND:GND:G::Ground pin";
    map[M + 12] = "GND:GND:G::Ground pin";
    map[N + 10] = "GND:GND:G::Ground pin";
    map[N + 13] = "GND:GND:G::Ground pin";
    map[N + 14] = "GND:GND:G::Ground pin";
    map[P + 10] = "GND:GND:G::Ground pin";
    map[P + 14] = "GND:GND:G::Ground pin";

    map[F + 5] = "VCC:SOC_IO_V33D:P::3.3V power for SOC";
    map[G + 5] = "VCC:SOC_IO_V33D:P::3.3V power for SOC";
    map[D + 10] = "VCC:SOC_IO_V33D_1:P::3.3V power for SOC";
    map[H + 5] = "VCC:SOC_CO_V12D:P::1.2V power for SOC";
    map[J + 5] = "VCC:SOC_CO_V12D:P::1.2V power for SOC";
    map[F + 9] = "VCC:SOC_CO_V12D:P::1.2V power for SOC";
    map[G + 9] = "VCC:SOC_CO_V12D:P::1.2V power for SOC";
    map[H + 9] = "VCC:SOC_CO_V12D:P::1.2V power for SOC";
    map[F + 10] = "VCC:SDRAM_IO_V33D:P::3.3V power for SDRAM";
    map[G + 10] = "VCC:SDRAM_IO_V33D:P::3.3V power for SDRAM";
    map[H + 10] = "VCC:SDRAM_IO_V33D:P::3.3V power for SDRAM";
    map[K + 5] = "VCC:EPHY_V33A:P::3.3V power for EPHY";
    map[K + 6] = "VCC:EPHY_V33A:P::3.3V power for EPHY";
    map[K + 7] = "VCC:EPHY_V33A:P::3.3V power for EPHY";
    map[K + 8] = "VCC:EPHY_V33A:P::3.3V power for EPHY";

    for(QMap<int, QString>::Iterator it = map.begin(); it != map.end(); it++) {
        QString text = it.value();
        QString item = QString("%1:%2:%3").arg(it.key()).arg(gmap[text.split(":").at(0)]).arg(text.split(":").at(1));
        pin.append(item);
    }

    update();
}

void MainWindow::paintEvent(QPaintEvent *)
{
    QPainter p(this);
    p.setRenderHint(QPainter::Antialiasing);

#ifdef Q_OS_MAC
    p.setFont(QFont("Andale Mono", 12));
#endif
#ifdef Q_OS_WIN
    p.setFont(QFont("Consola", 9));
#endif

    QTextOption o;
    o.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
    o.setAlignment(Qt::AlignCenter);

    foreach(QString info, pin) {
        QStringList d = info.split(":");
        // id:groupid:info
        int id = d[0].toInt(), gid = d[1].toInt();
        p.setBrush(QBrush(group[gid]));
        p.setPen(QPen(QBrush(QColor(200, 200, 200)), 2));

        QRectF rc;
        rc.setTopLeft(QPoint((id - 1) % CHIPX * PINWIDTH, (id - 1) / CHIPX * PINHEIGHT));
        rc.setSize(QSize(PINWIDTH, PINHEIGHT));
        p.drawRect(rc);

        p.setPen(QPen(QColor(255, 255, 255)));
        rc.adjust(4, 4, -4, -4);
        p.drawText(rc, d[2], o);
    }
}

void MainWindow::mousePressEvent(QMouseEvent *e)
{
    if(e->button() == Qt::LeftButton) {
        QPoint pt = e->pos();
        int x = pt.x() / PINWIDTH + 1;
        int y = pt.y() / PINHEIGHT;
        int index = y * CHIPX + x;
        QToolTip::showText(e->globalPos(), map[index]);
    }
}
